from batch_ingestion_client_py.main import BatchIngestor
from batch_ingestion_client_py.data import (
    Data,
    Entity,
    Claim,
    MainSnak,
    DataValue,
    ValueInLanguage
)
from batch_ingestion_client_py.response import (
    Response,
    Revision,
)



from batch_ingestion_client_py.data import Data
from batch_ingestion_client_py.response import Response
import requests
import json


class BatchIngestor:
    def __init__(
        self,
        base_url: str,
    ):
        route = "/w/rest.php/BatchIngestion/v0/batchcreate"
        self.url = base_url + route

    def ingest(self, data: Data):
        json_data = data.serialize()
        response = requests.post(self.url, json=json_data)
        success = response.status_code == 200
        if not success:
            raise Exception(response.text)
        json_res = json.loads(response.text)
        return Response.parse(json_res)



from typing import List, Dict


class Revision:
    def __init__(
        self,
        revision: int,
        id: str,
    ):
        self.revision = revision
        self.id = id

    def serialize(self):
        return {
            "revision": self.revision,
            "id": self.id,
        }

    @staticmethod
    def parse(data: Dict[str, str | int]):
        return Revision(
            revision=data["revision"],  # type: ignore
            id=data["id"],  # type: ignore
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()


class Response:
    def __init__(
        self,
        count: int,
        successes: int,
        response: List[Revision],
    ):
        self.count = count
        self.successes = successes
        self.response = response

    def serialize(self):
        return {
            "count": self.count,
            "successes": self.successes,
            "response": [r.serialize() for r in self.response],
        }

    @staticmethod
    def parse(data: Dict[str, int | List[Dict[str, str | int]]]):
        return Response(
            count=data["count"],  # type: ignore
            successes=data["successes"],  # type: ignore
            response=[
                Revision.parse(r)
                for r in data["response"]  # type: ignore
            ],
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()



from typing import List, Dict, Literal, Any


class ValueInLanguage:
    TYPE = Dict[str, str]

    def __init__(
        self,
        language: str,
        value: str,
    ):
        self.language = language
        self.value = value

    def serialize(self):
        return {
            "language": self.language,
            "value": self.value,
        }

    @staticmethod
    def parse(data: TYPE):
        return ValueInLanguage(
            language=data["language"],
            value=data["value"],
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()


class DataValue:
    TYPE = Dict[str, str]

    def __init__(
        self,
        value: str,
        type: str,
    ):
        self.value = value
        self.type = type

    def serialize(self):
        return {
            "value": self.value,
            "type": self.type,
        }

    @staticmethod
    def parse(data: TYPE):
        return DataValue(
            value=data["value"],
            type=data["type"],
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()


class MainSnak:
    TYPE = Dict[str, str | DataValue.TYPE]

    def __init__(
        self,
        snaktype: str,
        property: str,
        datatype: str,
        datavalue: DataValue,
    ):
        self.snaktype = snaktype
        self.property = property
        self.datatype = datatype
        self.datavalue = datavalue

    def serialize(self):
        return {
            "snaktype": self.snaktype,
            "property": self.property,
            "datatype": self.datatype,
            "datavalue": self.datavalue.serialize(),
        }

    @staticmethod
    def parse(data: TYPE):
        return MainSnak(
            snaktype=data["snaktype"],  # type: ignore
            property=data["property"],  # type: ignore
            datatype=data["datatype"],  # type: ignore
            datavalue=DataValue.parse(data["datavalue"]),  # type: ignore
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()


class Claim:
    TYPE = Dict[str, str | MainSnak.TYPE]

    def __init__(
        self,
        mainsnak: MainSnak,
        type: str,
        rank: str,
    ):
        self.mainsnak = mainsnak
        self.type = type
        self.rank = rank

    def serialize(self):
        return {
            "mainsnak": self.mainsnak.serialize(),
            "type": self.type,
            "rank": self.rank,
        }

    @staticmethod
    def parse(data: TYPE):
        return Claim(
            mainsnak=MainSnak.parse(data["mainsnak"]),  # type: ignore
            type=data["type"],  # type: ignore
            rank=data["rank"],  # type: ignore
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()


class Entity:
    TYPE = Dict[str, str | List[Claim.TYPE] | Dict[str, ValueInLanguage.TYPE]]

    def __init__(
        self,
        id: str,
        mode: Literal["add", "remove"] | None = None,
        type: str = "item",
        labels: Dict[str, ValueInLanguage] | None = None,
        descriptions: Dict[str, ValueInLanguage] | None = None,
        claims: Dict[str, List[Claim]] | None = None,
    ):
        self.id = id
        self.mode = mode
        self.type = type
        self.labels = labels
        self.descriptions = descriptions
        self.claims = claims

    def serialize(self):
        e = {
            "id": self.id,
            "mode": self.mode,
            "type": self.type,
        }
        if self.labels is not None:
            e["labels"] = {
                key: value.serialize()
                for key, value in self.labels.items()
            }
        if self.descriptions is not None:
            e["descriptions"] = {
                key: value.serialize()
                for key, value in self.descriptions.items()
            }
        if self.claims is not None:
            e["claims"] = {
                key: [claim.serialize() for claim in value]
                for key, value in self.claims.items()
            }
        return e

    @staticmethod
    def parse(data: TYPE):
        return Entity(
            id=data["id"],  # type: ignore
            mode=data["mode"],  # type: ignore
            type=data["type"],  # type: ignore
            labels={
                key: ValueInLanguage.parse(value)
                for key, value in data["labels"].items()  # type: ignore
            } if "labels" in data else None,
            descriptions={
                key: ValueInLanguage.parse(value)
                for key, value in data["descriptions"].items()  # type: ignore
            } if "descriptions" in data else None,
            claims={
                key: [Claim.parse(claim) for claim in value]  # type: ignore
                for key, value in data["claims"].items()  # type: ignore
            } if "claims" in data else None,
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()


class Data:
    TYPE = Dict[str, str | List[Entity.TYPE | Any]]

    def __init__(
        self,
        key: str,
        entities: List[Entity],
    ):
        self.key = key
        self.entities = entities

    def serialize(self):
        return {
            "key": self.key,
            "entities": [
                entity.serialize()
                for entity in self.entities
            ],
        }

    @staticmethod
    def parse(data: TYPE):
        return Data(
            key=data["key"],  # type: ignore
            entities=[
                Entity.parse(entity)  # type: ignore
                for entity in data["entities"]  # type: ignore
            ],
        )

    def __repr__(self) -> str:
        return self.serialize().__repr__()



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = "1.0.1"
DESCRIPTION = "A client for the BatchIngestion mediawiki API"

setup(
    name="batch_ingestion_client_py",
    version=VERSION,
    author="QuentinJanuel",
    author_email="<quentinjanuelkij@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=["requests"],
    keywords=[
        'python',
        'mediawiki',
        'batchingestion',
        'wikidata',
        'wikibase',
        'api',
        'client',
    ],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
